from django.conf.urls import url
from django.contrib.auth.views import LogoutView

from openid_connect_op.views.consent_view import ConsentView
from openid_connect_op.views.logout_request_view import LogoutRequestView
from .views.dynamic_registration_view import DynamicClientRegistrationView
from .views.jwks_view import JWKSView
from .views.userinfo_request_view import UserInfoView
from .views.wellknown import WellKnownView
from .views.authentication_request_view import AuthenticationRequestView
from .views.token_request_view import TokenRequestView

urlpatterns = [
    url('^openid/authorize', AuthenticationRequestView.as_view(), name='authorize'),
    url('^openid/token', TokenRequestView.as_view(), name='token'),
    url('^openid/userinfo', UserInfoView.as_view(), name='userinfo'),
    url('^openid/register', DynamicClientRegistrationView.as_view(), name='register'),
    url('^openid/jwks', JWKSView.as_view(), name='jwks'),
    url('^openid/logout', LogoutRequestView.as_view(), name='logout'),
    url('^openid/consent/(?P<client_id>\d+)/$', ConsentView.as_view(), name='consent'),
    url('^.well-known/openid-configuration', WellKnownView.as_view(), name='wellknown')
]

app_name = 'openid_connect_op'



from openid_connect_op.utils.params import Parameters, ParameterType


class AuthenticationParameters(Parameters):
    parameter_definitions = (
        ('redirect_uri', Parameters.REQUIRED),
        ('response_type', ParameterType(required=True, container_type=set, allowed_values={'code', 'token', 'id_token'})),
        ('client_id', Parameters.REQUIRED),
        ('scope', ParameterType(required=True, container_type=set)),

        ('state', Parameters.OPTIONAL),
        # currently not used at all
        ('nonce', Parameters.OPTIONAL),
        ('max_age', Parameters.OPTIONAL),
        # currently not used at all
        # 'id_token_hint', Parameters.OPTIONAL,
        # 'login_hint', Parameters.OPTIONAL,
        ('response_mode', ParameterType(required=False, container_type=set,
                                        allowed_values={'query', 'fragment', 'form_post'})),
        # currently not used at all
        # 'display', ParameterType(required=False, container_type=set,
        #                          allowed_values={'page', 'popup', 'touch', 'wap'}),
        ('prompt', ParameterType(required=False, container_type=set,
                                allowed_values={'none', 'login', 'consent', 'select_account'})),
        # currently not used at all
        # 'ui_locales', ParameterType(required=False, container_type=list),
        # 'acr_values', ParameterType(required=False, container_type=list),

        # extra parameters which are provided by AuthenticationRequestView
        ('username', Parameters.OPTIONAL),

        # openid claims
        ('claims', ParameterType(required=False, container_type=set)),
    )


class TokenParameters(Parameters):
    parameter_definitions = (
        ('grant_type', ParameterType(required=True, container_type=set,
                                    allowed_values={'authorization_code', 'refresh_token'})),
        # authorization_code
        ('code', Parameters.OPTIONAL), # required for authorization_code
        ('redirect_uri', Parameters.OPTIONAL),

        # common for POST credentials instead of basic auth
        ('client_id', Parameters.OPTIONAL),
        ('client_secret', Parameters.OPTIONAL),

        # refresh token
        ('refresh_token', Parameters.OPTIONAL),  # required for refresh_token
        ('scope', Parameters.OPTIONAL)
    )


class DynamicClientRegistrationParameters(Parameters):
    parameter_definitions = (
        ('redirect_uris', ParameterType(required=True, container_type=list)),
        ('response_types', ParameterType(required=False, container_type=set,
                                    allowed_values={'code', 'token', 'id_token'})),
        ('grant_types', ParameterType(required=False, container_type=set,
                                    allowed_values={'authorization_code', 'implicit', 'refresh_token'})),
        ('application_type', ParameterType(required=False, container_type=set,
                                    allowed_values={'web', 'native'})),
        ('contacts', ParameterType(required=False, container_type=set)),
        ('client_name', Parameters.OPTIONAL),
        ('logo_uri', Parameters.OPTIONAL),
        ('client_uri', Parameters.OPTIONAL),
        ('policy_uri', Parameters.OPTIONAL),
        ('tos_uri', Parameters.OPTIONAL),         # terms of service
        ('jwks_uri', Parameters.OPTIONAL),
        ('jwks', Parameters.OPTIONAL),
        ('sector_identifier_uri', Parameters.OPTIONAL),
        ('subject_type', Parameters.OPTIONAL),
        ('id_token_signed_response_alg', Parameters.OPTIONAL),
        ('id_token_encrypted_response_alg', Parameters.OPTIONAL),
        ('id_token_encrypted_response_enc', Parameters.OPTIONAL),
        ('userinfo_signed_response_alg', Parameters.OPTIONAL),
        ('userinfo_encrypted_response_alg', Parameters.OPTIONAL),
        ('userinfo_encrypted_response_enc', Parameters.OPTIONAL),
        ('request_object_signing_alg', Parameters.OPTIONAL),
        ('request_object_encryption_alg', Parameters.OPTIONAL),
        ('request_object_encryption_enc', Parameters.OPTIONAL),
        ('token_endpoint_auth_method', Parameters.OPTIONAL),
        ('token_endpoint_auth_signing_alg', Parameters.OPTIONAL),
        ('default_max_age', Parameters.OPTIONAL),
        ('require_auth_time', Parameters.OPTIONAL),
        ('default_acr_values', ParameterType(required=False, container_type=set)),
        ('initiate_login_uri', Parameters.OPTIONAL),
        ('request_uris', ParameterType(required=False, container_type=list)),
    )


import django.dispatch

before_user_consent = django.dispatch.Signal(providing_args=["openid_client", "user", "consent_already_signed"])
"""
A signal that is sent after the user is logged in but before the consent screen is shown. The signal is sent even
when the consent step is skipped, if it was signed previously. In that case, consent_already_signed is set to True.
If the signal listener returns an HttpResponse, this response is returned to the user agent instead of the standard 
openid redirection response. For example, user can be directed to a page that states that he/she has no rights 
to access the application.
"""


after_user_consent = django.dispatch.Signal(providing_args=["openid_client", "user"])
"""
A signal that is sent after the user has accepted consent. If the consent has been accepted in a previous login, 
this signal is not sent. If the signal listener
returns an HttpResponse, this response is returned to the user agent instead of the standard openid redirection
response. For example, user can be directed to a page that states that he/she has no rights to access the application.
"""



import base64
import traceback

from django.conf import settings
from django.contrib.auth.models import User
from django.http import HttpResponseBadRequest
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from ratelimit.mixins import RatelimitMixin

from openid_connect_op.models import OpenIDClient, OpenIDKey
from openid_connect_op.utils.jwt import JWTTools
from . import OAuthRequestMixin
from .errors import OAuthError
from .parameters import AuthenticationParameters, TokenParameters
from ..models import OpenIDToken


# section 4.1.3 of OAUTH 2.0
class TokenRequestView(OAuthRequestMixin, RatelimitMixin, View):
    ratelimit_key = 'ip'
    ratelimit_rate = '10/m'
    ratelimit_block = True
    ratelimit_method = 'ALL'
    use_redirect_uri = False
    attribute_parsing_error = 'invalid_request'

    # noinspection PyAttributeOutsideInit
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        if request.method not in ('GET', 'POST'):
            return HttpResponseBadRequest('Only GET or POST are supported on OpenID endpoint')

        # noinspection PyBroadException
        try:
            self.parse_request_parameters(request, TokenParameters)
            try:
                self.request_parameters.check_errors()
            except AttributeError as e:
                raise OAuthError(error=self.attribute_parsing_error, error_description=str(e))

            client = self.authenticate_client(request)

            if self.request_parameters.grant_type == {'authorization_code'}:
                return self.process_authorization_code_grant_type(request, client)
            elif self.request_parameters.grant_type == {'refresh_token'}:
                return self.process_refresh_token(request, client)
            else:
                raise OAuthError(error='invalid_request',
                                 error_description='Invalid grant type %s' % self.request_parameters.grant_type)

        except OAuthError as err:
            return self.oauth_send_answer(request, {
                'error': err.error,
                'error_description': err.error_description
            })
        except BaseException as err:
            traceback.print_exc()
            return self.oauth_send_answer(request, {
                'error': 'unknown_error',
                'error_description': 'Unknown error occurred at %s, check the logs' % timezone.now()
            })

    def process_authorization_code_grant_type(self, request, client):

        if not self.request_parameters.code:
            raise OAuthError(error='invalid_request',
                             error_description='Required parameter with name "code" is not present')

        authorization_token = OpenIDToken.objects.filter(
            token_hash=OpenIDToken.get_token_hash(self.request_parameters.code),
            client=client,
            token_type=OpenIDToken.TOKEN_TYPE_AUTH).first()

        if not authorization_token:
            raise OAuthError(error='unauthorized_client',
                             error_description='Authorization token not found')

        if authorization_token.expired:
            raise OAuthError(error='unauthorized_client',
                             error_description='Authorization token expired')

        authentication_parameters = AuthenticationParameters(authorization_token.token_data)

        # prevent reusing
        authorization_token.delete()

        self.validate_redirect_uri(authentication_parameters)

        return self.generate_tokens_and_oauth_response(authentication_parameters, client, request)

    def process_refresh_token(self, request, client):
        if not self.request_parameters.refresh_token:
            raise OAuthError(error='invalid_request',
                             error_description='Required parameter with name "refresh_token" is not present')
        try:
            refresh_token = OpenIDToken.objects.get(
                token_hash=OpenIDToken.get_token_hash(self.request_parameters.refresh_token))
            if refresh_token.expired:
                raise OAuthError(error='invalid_grant', error_description='Refresh token expired')
        except OpenIDToken.DoesNotExist:
            raise OAuthError(error='invalid_grant', error_description='No such token was found')

        original_access_token = refresh_token.root_token
        authentication_parameters = AuthenticationParameters(original_access_token.token_data)
        # noinspection PyTypeChecker
        return self.generate_tokens_and_oauth_response(authentication_parameters, client, request)

    def generate_tokens_and_oauth_response(self, authentication_parameters, client, request):
        access_token_ttl = getattr(settings, 'OPENID_DEFAULT_ACCESS_TOKEN_TTL', 3600)
        refresh_token_ttl = getattr(settings, 'OPENID_DEFAULT_REFRESH_TOKEN_TTL', 3600 * 10)
        user = User.objects.get(username=authentication_parameters.username)

        access_token, db_access_token = OpenIDToken.create_token(
            client,
            OpenIDToken.TOKEN_TYPE_ACCESS_BEARER_TOKEN,
            authentication_parameters.to_dict(),
            access_token_ttl,
            user,
        )
        refresh_token, refresh_db_token = OpenIDToken.create_token(
            client,
            OpenIDToken.TOKEN_TYPE_REFRESH_TOKEN,
            {},
            refresh_token_ttl,
            user,
            root_db_token=db_access_token
        )
        id_token = self.create_id_token(request, client, authentication_parameters, db_access_token, user)
        return self.oauth_send_answer(request, {
            'access_token': access_token,
            'token_type': 'Bearer',
            'refresh_token': refresh_token,
            'expires_in': access_token_ttl,
            'id_token': id_token
        })

    def validate_redirect_uri(self, authentication_parameters):
        auth_redirect_uri = authentication_parameters.redirect_uri
        token_redirect_uri = self.request_parameters.redirect_uri
        if auth_redirect_uri and auth_redirect_uri != token_redirect_uri:
            raise OAuthError(error='invalid_request',
                             error_description='redirect_uri does not match the one used in /authorize endpoint')
        if not auth_redirect_uri and token_redirect_uri:
            raise OAuthError(error='invalid_request',
                             error_description='redirect_uri not used in authentication but passed for token')

    def authenticate_client(self, request):
        auth_header = request.META.get('HTTP_AUTHORIZATION')
        if auth_header and auth_header.startswith('Basic '):
            return self.authenticate_with_http_basic(auth_header)
        if self.request_parameters.client_secret:
            return self.authenticate_with_client_secret()
        if self.request_parameters.client_id:
            client = self.try_null_authentication()
            if client:
                return client
        raise OAuthError(error='unsupported_authentication_method',
                         error_description='Only HTTP Basic auth or client_secret is supported')

    def authenticate_with_http_basic(self, auth_header):
        username, password = base64.b64decode(auth_header[6:].strip()).decode('utf-8').split(':', 1)
        try:
            client = OpenIDClient.objects.get(client_id=username)
            if not client.check_client_secret(password):
                raise OAuthError(error='unauthorized_client', error_description='Bad username or password')

            if self.request_parameters.client_id and client.client_id != self.request_parameters.client_id:
                raise OAuthError(error='invalid_request',
                                 error_description='client_id does not match with authorized client')

            if client.client_auth_type != client.CLIENT_AUTH_TYPE_BASIC:
                raise OAuthError(error='invalid_request',
                                 error_description='Used HTTP Basic but client configured to different auth')
            return client

        except OpenIDClient.DoesNotExist:
            raise OAuthError(error='unauthorized_client', error_description='Bad username or password')

    def authenticate_with_client_secret(self):
        if not self.request_parameters.client_id:
            raise OAuthError(error='invalid_request',
                             error_description='Need client_id when using client_secret')
        try:
            client = OpenIDClient.objects.get(client_id=self.request_parameters.client_id)

            if client.client_auth_type != client.CLIENT_AUTH_TYPE_POST:
                raise OAuthError(error='invalid_request',
                                 error_description='Client not configured to use POST authentication')

            if client.check_client_secret(self.request_parameters.client_secret):
                return client

        except OpenIDClient.DoesNotExist:
            pass

        raise OAuthError(error='unauthorized_client', error_description='Bad client_id or client_secret')

    def try_null_authentication(self):
        if not self.request_parameters.client_id:
            return None

        try:
            client = OpenIDClient.objects.get(client_id=self.request_parameters.client_id)

            if client.client_auth_type != client.CLIENT_AUTH_TYPE_NONE:
                return None

            return client
        except OpenIDClient.DoesNotExist:
            pass

    @staticmethod
    def create_id_token(request, client, authentication_parameters, db_access_token, user):
        id_token = {
            "iss": request.build_absolute_uri('/'),
            "sub": client.make_sub(request.user.username),
            "aud": [client.client_id],
            "exp": int(db_access_token.expiration.timestamp()),
            # the time at which user was authenticated - we do not have this stored anywhere ...
            # "auth_time": 1311280969,

            # level of trustability of the login
            # "acr": "urn:mace:incommon:iap:silver",

            # names of authentication methods that were used to login this user
            # "amr": None,

            # the audience is the same as the acceptor of this token, so omitting the azp
            # "azp": None
        }

        if authentication_parameters.nonce:
            id_token['nonce'] = authentication_parameters.nonce

        token = JWTTools.generate_jwt(id_token)

        # save the token to the database
        OpenIDToken.create_token(client, OpenIDToken.TOKEN_TYPE_ID_TOKEN, {
            'token': token
        }, db_access_token.expiration, user, db_access_token, token=token)

        return token



import base64
import hmac
import struct
import time

import zlib
from Cryptodome.Cipher import AES


class CryptoTools:

    @staticmethod
    def encrypt(message, ttl=None, not_valid_before=None, key=None, prefix=b'', zlib_dict=b''):
        """
        Encrypts a message and puts in timestamp

        :param message:     an instance of bytes
        :param ttl:         time to live in seconds
        :param not_valid_before:  integer, number of seconds since epoch as given by time.time()
        :param key:         an array of 16 bytes, if not set use settings.OPENID_CONNECT_OP_AES_KEY
        :param prefix:      add this prefix to the message before encryption, during decrypt the message will be checked for it
        :return:            a timestamped, encrypted and signed message
        """
        if b':' in prefix:
            raise AttributeError('Prefix must not contain character :')

        if ttl is not None:
            if not_valid_before is None:
                not_valid_before = int(time.time())

        if ttl is None:
            not_valid_after = 2 ** 64 - 1
        else:
            not_valid_after = not_valid_before + ttl

        not_valid_after  = struct.pack('>Q', not_valid_after)

        message = not_valid_after + prefix + b':' + message

        co = zlib.compressobj(level=9, wbits=-9, zdict=zlib_dict)
        message = co.compress(message) + co.flush()

        if key is None:
            raise AttributeError('Key can not be None')
        assert len(key) == 16

        cipher = AES.new(key, AES.MODE_GCM)
        cipher_text, tag = cipher.encrypt_and_digest(message)
        encrypted_message = b''.join([cipher.nonce, tag, cipher_text])
        ret = base64.urlsafe_b64encode(encrypted_message)
        while ret[-1] == ord(b'='):
            ret = ret[:-1]
        return ret

    @staticmethod
    def decrypt(message, check_ttl=True, key=None, expected_prefix=b'', zlib_dict=b''):
        """
        Decrypts a message and checks that it has not yet expired

        :param message:     the encrypted message
        :param check_ttl:   if true, check its ttl, if false, ignore ttl at all
        :param key:         an array of 16 bytes, if not set use settings.OPENID_CONNECT_OP_AES_KEY
        :return:
        :param expected_prefix: the message will be checked against this prefix and if it does not match, an AttributeError will be raised
        """
        if key is None:
            raise AttributeError('Key can not be None')

        if len(message) % 4 == 2:
            message += b'=='
        elif len(message) % 4 == 3:
            message += b'='
        encrypted_message = base64.urlsafe_b64decode(message)
        # nonce and tag are always 16 bytes long
        nonce, tag, cipher_text = encrypted_message[:16], encrypted_message[16:32], encrypted_message[32:]
        cipher = AES.new(key, AES.MODE_GCM, nonce=nonce)
        message = cipher.decrypt_and_verify(cipher_text, tag)

        do = zlib.decompressobj(wbits=-9, zdict=zlib_dict)
        message = do.decompress(message) + do.flush()

        not_valid_after = message[:8]
        message = message[8:]

        if check_ttl:
            not_valid_after  = struct.unpack('>Q', not_valid_after)[0]
            current_time = time.time()
            if current_time > not_valid_after:
                raise AttributeError('The message has expired')

        prefix, message = message.split(b':', maxsplit=1)
        if not hmac.compare_digest(prefix, expected_prefix):
            # use hmac to prevent timing attacks
            raise AttributeError('Expecting prefix %s, got %s' % (expected_prefix, prefix))

        return message



import datetime
from functools import lru_cache

import python_jwt as jwt
import jwcrypto.jwk as jwk
from django.conf import settings


class JWTTools:

    @staticmethod
    def generate_jwt(payload):
        from openid_connect_op.models import OpenIDKey, OpenIDClient
        cipher = getattr(settings, 'OPENID_JWT_CIPHER', 'RS256')
        return jwt.generate_jwt(payload,
                                jwk.JWK.from_pem(OpenIDClient.self_instance().get_key(OpenIDKey.JWK_RSA_PRIVATE_KEY)),
                                cipher)

    @staticmethod
    def validate_jwt(token):
        from openid_connect_op.models import OpenIDKey, OpenIDClient
        cipher = getattr(settings, 'OPENID_JWT_CIPHER', 'RS256')
        return jwt.verify_jwt(token,
                              jwk.JWK.from_pem(OpenIDClient.self_instance().get_key(OpenIDKey.JWK_RSA_PUBLIC_KEY)),
                              [cipher])

    @staticmethod
    def get_jwks():
        from openid_connect_op.models import OpenIDKey, OpenIDClient
        return jwk.JWK.from_pem(OpenIDClient.self_instance().get_key(OpenIDKey.JWK_RSA_PUBLIC_KEY)).export_public()




# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-02 08:08
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='OpenIDAgreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('text_en', models.TextField(null=True)),
                ('text_cs', models.TextField(null=True)),
                ('obligatory', models.BooleanField()),
                ('allowed_scopes', jsonfield.fields.JSONField(blank=True, null=True, verbose_name='List of allowed scopes for this agreement.')),
                ('allowed_claims', jsonfield.fields.JSONField(blank=True, null=True, verbose_name='List of allowed claims for this agreement')),
                ('username_auto_agreement_regexp', models.CharField(blank=True, max_length=256, null=True, verbose_name='Usernames matching this regexp will have this agreement automatically agreed to')),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDClient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('client_id', models.CharField(max_length=128, unique=True)),
                ('redirect_uris', models.TextField(default='')),
                ('client_auth_type', models.CharField(choices=[('basic', 'Basic Authentication'), ('post', 'Authentication data in POST request'), ('sjwt', 'JSON Web token with pre-shared secret'), ('pkjwt', 'JSON Web token with public/private key'), ('none', 'No client authentication performed')], max_length=8)),
                ('client_hashed_secret', models.CharField(max_length=128)),
                ('client_name', models.CharField(max_length=128)),
                ('client_name_en', models.CharField(max_length=128, null=True)),
                ('client_name_cs', models.CharField(max_length=128, null=True)),
                ('allowed_scopes', jsonfield.fields.JSONField(blank=True, null=True, verbose_name='List of allowed scopes. If None, all scopes are allowed')),
                ('allowed_claims', jsonfield.fields.JSONField(blank=True, null=True, verbose_name='List of allowed claims. If None, all claims from allowed scopes are returned')),
                ('sub_hash', models.CharField(blank=True, max_length=256, null=True, verbose_name='If set, <<sub>> values (that is, username) will be concatenated with this value and sha256')),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key_type', models.CharField(choices=[('jwk_rsa_private', 'JWK RSA private key'), ('jwk_rsa_public', 'JWK RSA public key'), ('aes', 'AES key')], max_length=16)),
                ('encrypted_key_value', models.BinaryField()),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openid_connect_op.OpenIDClient')),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token_hash', models.CharField(max_length=64, unique=True)),
                ('token_type', models.CharField(max_length=4)),
                ('token_data', jsonfield.fields.JSONField(default={})),
                ('expiration', models.DateTimeField()),
                ('client', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openid_connect_op.OpenIDClient')),
                ('root_token', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='related_tokens', to='openid_connect_op.OpenIDToken')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='OpenIDUserAgreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('agreed_on', models.DateTimeField()),
                ('agreed_by_user', models.BooleanField(default=False, verbose_name='If set to False, the agreement was created automatically, otherwise it was created explicitly by user')),
                ('agreement', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_agreements', to='openid_connect_op.OpenIDAgreement')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='user_agreements', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='openidagreement',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='agreements', to='openid_connect_op.OpenIDClient'),
        ),
    ]



from openid_connect_op.utils.crypto import CryptoTools

try:
    import secrets
except ImportError:
    import openid_connect_op.utils.secrets_backport as secrets

import jwcrypto.jwk as jwk
from django.conf import settings
from django.core.management import BaseCommand

from openid_connect_op.models import OpenIDClient, OpenIDKey


class Command(BaseCommand):
    def handle(self, *args, **kwargs):
        """
        Creates JWT keys at the location pointed by settings.OPENID_JWT_PRIVATE_KEY, settings.OPENID_JWT_PUBLIC_KEY
        """
        key = jwk.JWK.generate(kty='RSA', size=2048)
        priv_pem = key.export_to_pem(private_key=True, password=None)
        pub_pem = key.export_to_pem()
        aes = secrets.token_urlsafe(nbytes=16).encode('ascii')[:16]

        client = OpenIDClient.objects.get_or_create(client_id=OpenIDClient.SELF_CLIENT_ID,
                                                    defaults={
                                                        'client_auth_type': OpenIDClient.CLIENT_AUTH_TYPE_INVALID,
                                                        'client_name': 'This server'
                                                    })[0]

        OpenIDKey.objects.update_or_create(client=client, key_type=OpenIDKey.JWK_RSA_PUBLIC_KEY, defaults={
            'encrypted_key_value': OpenIDKey.encrypt_key(pub_pem)
        })

        OpenIDKey.objects.update_or_create(client=client, key_type=OpenIDKey.JWK_RSA_PRIVATE_KEY, defaults={
            'encrypted_key_value': OpenIDKey.encrypt_key(priv_pem)
        })

        OpenIDKey.objects.update_or_create(client=client, key_type=OpenIDKey.AES_KEY, defaults={
            'encrypted_key_value': OpenIDKey.encrypt_key(aes)
        })



import pytest
from django.contrib.auth.models import User
from django.utils import timezone

from openid_connect_op.models import OpenIDClient, OpenIDAgreement, OpenIDUserAgreement


@pytest.mark.django_db
class TestAgreements:

    def test_obligatory_agreements(self):
        client_config = OpenIDClient.objects.create(redirect_uris='http://my-site.com/auth/complete')
        agr1 = OpenIDAgreement.objects.create(client=client_config, text='Lorem ipsum', obligatory=True)
        agr2 = OpenIDAgreement.objects.create(client=client_config, text='Dolor sit amet', obligatory=True)

        u = User.objects.create(username='aa@vscht.cz')
        assert not client_config.has_user_agreement(u)

        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == {agr1, agr2}

        OpenIDUserAgreement.objects.create(agreement=agr1, user=u, agreed_on=timezone.now())

        assert not client_config.has_user_agreement(u)
        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == {agr2}

        OpenIDUserAgreement.objects.create(agreement=agr2, user=u, agreed_on=timezone.now())

        assert client_config.has_user_agreement(u)
        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == set()

    def test_optional_agreements(self):
        client_config = OpenIDClient.objects.create(redirect_uris='http://my-site.com/auth/complete')
        agr1 = OpenIDAgreement.objects.create(client=client_config, text='Lorem ipsum', obligatory=False)
        agr2 = OpenIDAgreement.objects.create(client=client_config, text='Dolor sit amet', obligatory=False)

        u = User.objects.create(username='aa@vscht.cz')
        assert client_config.has_user_agreement(u)

        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == {agr1, agr2}

        OpenIDUserAgreement.objects.create(agreement=agr1, user=u, agreed_on=timezone.now())

        assert client_config.has_user_agreement(u)
        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == {agr2}

        OpenIDUserAgreement.objects.create(agreement=agr2, user=u, agreed_on=timezone.now())

        assert client_config.has_user_agreement(u)
        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == set()

    def test_auto_approved_agreements(self):
        client_config = OpenIDClient.objects.create(redirect_uris='http://my-site.com/auth/complete')
        agr1 = OpenIDAgreement.objects.create(client=client_config, text='Lorem ipsum', obligatory=True,
                                              username_auto_agreement_regexp='^.*@vscht.cz$')

        u = User.objects.create(username='aa@vscht.cz')
        assert client_config.has_user_agreement(u)

        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == set()

    def test_auto_approved_agreements2(self):
        client_config = OpenIDClient.objects.create(redirect_uris='http://my-site.com/auth/complete')
        agr1 = OpenIDAgreement.objects.create(client=client_config, text='Lorem ipsum', obligatory=True,
                                              username_auto_agreement_regexp='^.*@vscht.cz$')

        u = User.objects.create(username='aa@gmail.com')
        assert not client_config.has_user_agreement(u)

        unsigned_agreements = set(client_config.get_unsigned_agreements(u))
        assert unsigned_agreements == {agr1}



"""
This file was taken from python 3.6
"""

"""Generate cryptographically strong pseudo-random numbers suitable for
managing secrets such as account authentication, tokens, and similar.

See PEP 506 for more information.
https://www.python.org/dev/peps/pep-0506/

"""

__all__ = ['choice', 'randbits', 'SystemRandom',
           'token_bytes', 'token_urlsafe',
           'compare_digest',
           ]

import base64
import os

from hmac import compare_digest
from random import SystemRandom

_sysrand = SystemRandom()

randbits = _sysrand.getrandbits
choice = _sysrand.choice


def token_bytes(nbytes):
    """Return a random byte string containing *nbytes* bytes.

    If *nbytes* is ``None`` or not supplied, a reasonable
    default is used.

    >>> token_bytes(16)  #doctest:+SKIP
    b'\\xebr\\x17D*t\\xae\\xd4\\xe3S\\xb6\\xe2\\xebP1\\x8b'

    """
    return os.urandom(nbytes)


def token_urlsafe(nbytes):
    """Return a random URL-safe text string, in Base64 encoding.

    The string has *nbytes* random bytes.  If *nbytes* is ``None``
    or not supplied, a reasonable default is used.

    >>> token_urlsafe(16)  #doctest:+SKIP
    'Drmhze6EPcv0fN_81Bj-nA'

    """
    tok = token_bytes(nbytes)
    return base64.urlsafe_b64encode(tok).rstrip(b'=').decode('ascii')



import base64
from functools import wraps

from django.conf import settings
from django.http.response import HttpResponseForbidden
from django.utils import timezone
from django.utils.decorators import available_attrs

from openid_connect_op.models import OpenIDToken


def get_access_token_from_auth_header(auth_header):
    auth_header = auth_header.strip()
    if not auth_header.startswith('Bearer '):
        raise AttributeError('Not a Bearer token')
    return auth_header[7:]


def get_access_token_from_post_data(request):
    post_data = request.POST
    if request.content_type != 'application/x-www-form-urlencoded':
        return None
    if 'access_token' not in post_data:
        return None
    return post_data['access_token']


def get_access_token_from_uri_query(get_data):
    if 'access_token' not in get_data:
        return None
    return get_data['access_token']


def extract_access_token(request, forbidden_on_not_present):
    auth_header = request.META.get('HTTP_AUTHORIZATION', None)
    access_token = None
    try:
        if auth_header is not None:
            access_token = get_access_token_from_auth_header(auth_header)
        if request.method == 'POST' and not access_token:
            access_token = get_access_token_from_post_data(request)
        if not access_token:
            access_token = get_access_token_from_uri_query(request.GET)
        if not access_token:
            if forbidden_on_not_present:
                return HttpResponseForbidden('No access token provided')
            else:
                return None
        try:
            db_access_token = OpenIDToken.objects.get(token_hash=OpenIDToken.get_token_hash(access_token))
            if db_access_token.expiration < timezone.now():
                return HttpResponseForbidden('Expired access token')
            return db_access_token
        except OpenIDToken.DoesNotExist:
            return HttpResponseForbidden('Provided access token not found')
    except BaseException as e:
        return HttpResponseForbidden('Access error: %s' % e)


def access_token_required(disabled_settings=None):
    def wrapper(func):
        """
        Check that access token is present on the request and is valid. If not, returns HttpResponseForbidden.
        request is annotated with the database access token, i.e. isinstance(req.openid_access_token, OpenIDToken) == True
        """
        @wraps(func, assigned=available_attrs(func))
        def inner(request, *args, **kwargs):
            if disabled_settings:
                if getattr(settings, disabled_settings, False):
                    request.openid_access_token = None
                    return func(request, *args, **kwargs)
            db_access_token = extract_access_token(request, True)
            if isinstance(db_access_token, HttpResponseForbidden):
                return db_access_token
            request.openid_access_token = db_access_token
            return func(request, *args, **kwargs)
        return inner
    return wrapper

__all__ = ('access_token_required', )



import json

import pytest
from django.core.management import call_command
import jwcrypto.jwk as jwk

@pytest.mark.django_db
class TestJWKS:

    @pytest.fixture(autouse=True)
    def init_jwk(self):
        call_command('create_jwt_keys')

    def test_get_jwks(self, client):
        resp = client.get('/openid/jwks')
        assert resp.status_code == 200
        data = json.loads(resp.content.decode('utf-8'))
        assert 'keys' in data
        assert len(data['keys']) >= 1
        for key in data['keys']:
            jwk.JWK(**key).export_public() == key




import inspect
import itertools

from django.utils.functional import cached_property
from django.utils.module_loading import import_string


class UserInfoProvider:
    """
    A provider for userinfo claims. To register custom providers, register them in settings file as:

        OPENID_CLAIM_PROVIDERS = ['myapp.openid_providers.MyProvider']
    """

    SCOPE_CLAIMS = {}
    """
    The scopes and claims that are handled by this userinfo provider. A dictionary of scope names -> list of claims
    within the scope. For custom scopes/claims consider using fully qualified names to avoid name collisions.
    
    Example: 
        SCOPE_CLAIMS = {
            'http://myorg.com#idm' : [
                'http://myorg.com#personal_number',
                'http://myorg.com#roles',
            ]
        }
    """

    def get_claims(self, db_access_token):
        """
        Returns userinfo claims.

        This call must be idempotent.

        :param db_access_token:     an instance of OpenIDToken of the user whose claims should be returned
        :return:                    a dictionary of claims.
        """
        return {}

    def get_scope_claims(self):
        return self.SCOPE_CLAIMS

#
# OpenID Connect spec: default scopes and claims:
#
#  profile:
#      name, family_name, given_name, middle_name, nickname, preferred_username,
#      profile, picture, website, gender, birthdate, zoneinfo, locale, and updated_at.
# email
#      email, email_verified
# address
#      address
# phone
#      phone_number, phone_number_verified
#


class DjangoProfileProvider(UserInfoProvider):
    SCOPE_CLAIMS = {
        'profile': ['name', 'family_name', 'given_name', 'preferred_username'],
        # sub is not in a named scope, it is always added
        '': ['sub']
    }

    def get_claims(self, db_access_token, scope_or_claim=None):
        user = db_access_token.user

        return {
            'name': user.get_full_name(),
            'family_name': user.last_name,
            'given_name': user.first_name,
            'preferred_username': user.username,
            'sub': db_access_token.client.make_sub(user.username),
            # 'middle_name' not set on django user,
            # 'nickname' not set on django user,
            # 'profile' not set on django user,
            # 'picture' not set on django user,
            # 'website' not set on django user,
            # 'gender' not set on django user,
            # 'birthdate' not set on django user,
            # 'zoneinfo' not set on django user,
            # 'locale' not set on django user,
            # 'updated_at' not set on django user.
        }


class DjangoEmailProvider(UserInfoProvider):
    SCOPE_CLAIMS = {
        'email': ['email']
    }

    def get_claims(self, db_access_token, scope_or_claim=None):
        user = db_access_token.user

        return {
            'email': user.email
            # 'email_verified' not set on django user,
        }


class UserInfoProviderRegistry:
    """
    registry of UserInfoProvider instances
    """

    DEFAULT_CLAIM_PROVIDERS = [
        DjangoProfileProvider,
        DjangoEmailProvider
    ]

    def __init__(self, user_claim_providers):
        self.scope_claims = {}
        self.claim_providers = {}
        for provider in itertools.chain(user_claim_providers, self.DEFAULT_CLAIM_PROVIDERS):
            self.register_provider(provider)

    def register_provider(self, provider_class_or_classname_or_instance):
        if isinstance(provider_class_or_classname_or_instance, str):
            provider = import_string(provider_class_or_classname_or_instance)()
        elif inspect.isclass(provider_class_or_classname_or_instance):
            provider = provider_class_or_classname_or_instance()
        else:
            provider = provider_class_or_classname_or_instance

        for scope_name, claims in provider.get_scope_claims().items():
            if scope_name not in self.scope_claims:
                self.scope_claims[scope_name] = []
            for claim in claims:
                if claim not in self.scope_claims[scope_name]:
                    self.scope_claims[scope_name].append(claim)
                if claim not in self.claim_providers:
                    self.claim_providers[claim] = []
                self.claim_providers[claim].append(provider)

    @cached_property
    def supported_scopes(self):
        return [scope for scope in sorted(self.scope_claims.keys()) if scope]

    @cached_property
    def supported_claims(self):
        return list(sorted(self.claim_providers.keys()))

    def get_claims(self, db_access_token, scopes, claims):
        claim_values = {}
        claim_names = set(claims)
        for scope in scopes:
            claim_names.update(self.scope_claims.get(scope, []))
        # add all claims from the unnamed scope
        claim_names.update(self.scope_claims[''])

        cache = {}
        for claim in self._filter_claims(claim_names, db_access_token.client):
            if claim in self.claim_providers:
                for provider in self.claim_providers[claim]:
                    if provider not in cache:
                        provider_data = provider.get_claims(db_access_token, claim)
                        cache[provider] = provider_data
                    else:
                        provider_data = cache[provider]

                    if provider_data is not None:
                        if claim not in provider_data:
                            raise ValueError('Provider "%s" registered for claim "%s" has returned different claim!')
                        if provider_data[claim]:
                            # set only claim with value
                            claim_values[claim] = provider_data[claim]
                        break

        return claim_values

    def _filter_claims(self, claims, client):
        """filter claims to only those that are allowed for the client"""
        allowed_claims = client.allowed_claims
        if allowed_claims is None:
            allowed_scopes = client.allowed_scopes
            if allowed_scopes is not None:
                allowed_claims = set()
                for scope in allowed_scopes:
                    if scope in self.scope_claims:
                        allowed_claims.update(self.scope_claims[scope])
        else:
            allowed_claims = set(allowed_claims)

        # always return empty scope as well if claims are restricted
        if allowed_claims:
            allowed_claims.update(self.scope_claims[''])

        for claim in claims:
            if not allowed_claims or claim in allowed_claims:
                yield claim







from django.conf import settings
from django.http.response import JsonResponse
from django.urls import reverse
from django.views import View


class WellKnownView(View):
    defaults = {
        "token_endpoint_auth_methods_supported":
            ["client_secret_basic"],
        "response_types_supported":
            ["code"],
        "subject_types_supported":
            ["public", "pairwise"],
        "userinfo_signing_alg_values_supported":
            ["RS256"],
        "id_token_signing_alg_values_supported":
            ["RS256"],
        "display_values_supported":
            ["page"],
        "claim_types_supported":
            ["normal"],
        "claims_parameter_supported":
            False,
        "ui_locales_supported":
            ["en-US", "cs-CZ"]
    }
    extra = {}

    def get(self, request, *args, **kwargs):
        resp = {}
        resp.update(self.defaults)
        resp.update(self.extra)
        resp['issuer'] = request.build_absolute_uri('/')
        resp['authorization_endpoint'] = request.build_absolute_uri(reverse('openid_connect_op:authorize'))
        resp['token_endpoint'] = request.build_absolute_uri(reverse('openid_connect_op:token'))
        resp['userinfo_endpoint'] = request.build_absolute_uri(reverse('openid_connect_op:userinfo'))
        resp['jwks_uri'] = request.build_absolute_uri(reverse('openid_connect_op:jwks'))
        resp['registration_endpoint'] = request.build_absolute_uri(reverse('openid_connect_op:register'))
        resp['end_session_endpoint'] = request.build_absolute_uri(reverse('openid_connect_op:logout'))

        resp["scopes_supported"] = ['openid'] + settings.OPENID_USERINFO_PROVIDERS.supported_scopes
        resp["claims_supported"] = settings.OPENID_USERINFO_PROVIDERS.supported_claims + ['iss', 'auth_time', 'acr']

        return JsonResponse(resp)



import json
from urllib.parse import urlencode

from django.conf import settings
from django.http import JsonResponse, HttpResponseRedirect
from django.shortcuts import render
from django.utils.cache import add_never_cache_headers
from django.utils.functional import cached_property

from openid_connect_op.views.errors import OAuthError
from openid_connect_op.views.parameters import TokenParameters


class OAuthRequestMixin:
    request_parameters = None
    use_redirect_uri = True
    attribute_parsing_error = 'invalid_request'

    def oauth_send_answer(self, request, response_params):
        actual_params = {}
        actual_params.update(response_params)
        redirect_uri = getattr(self.request_parameters, 'redirect_uri', None) or \
                       request.GET.get('redirect_uri', None) or \
                       request.POST.get('redirect_uri', None)
        if hasattr(self.request_parameters, 'state') and self.request_parameters.state:
            actual_params['state'] = self.request_parameters.state

        if hasattr(self.request_parameters, 'response_mode') and 'form_post' in self.request_parameters.response_mode:
            resp = render(request, 'django-open-id/form_post_response_mode.html', {
                'params': actual_params,
                'redirect_uri': redirect_uri
            })
            add_never_cache_headers(resp)
            return resp

        if not redirect_uri or not self.use_redirect_uri:
            status = 200
            if 'error' in response_params:
                if response_params['error'] == 'unauthorized_client':
                    status = 403
                else:
                    status = 400
            return JsonResponse(actual_params, status=status)

        if '?' in redirect_uri:
            redirect_uri += '&'
        else:
            redirect_uri += '?'
        redirect_uri += urlencode(actual_params)
        return HttpResponseRedirect(redirect_uri)

    def parse_request_parameters(self, request, parameters_class):
        try:
            if request.method == 'GET':
                params = {k: v for k, v in request.GET.items()}
            else:
                params = {}
                if request.GET:
                    params.update({k: v for k, v in request.GET.items()})
                if request.POST:
                    params.update({k: v for k, v in request.POST.items()})
                if request.content_type == 'application/json':
                    params.update(json.loads(request.body.decode('utf-8')))

            # noinspection PyAttributeOutsideInit
            self.request_parameters = parameters_class(params)
        except AttributeError as e:
            raise OAuthError(error=self.attribute_parsing_error, error_description=str(e))



from django.conf import settings
from social_core.backends.open_id_connect import OpenIdConnectAuth


class ConfiguredOpenIdConnectAuth(OpenIdConnectAuth):
    OIDC_ENDPOINT = settings.SERVER_URL
    STATE_PARAMETER = False
    REDIRECT_STATE = False
    name = 'test'

    def get_or_create_state(self):
        return '123'


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

from openid_connect_op.userinfo_providers import UserInfoProviderRegistry


class OpenIDConnectOPApp(AppConfig):
    name = 'openid_connect_op'
    verbose_name = _('OpenID Connect Provider')

    def ready(self):
        from django.conf import settings

        if not getattr(settings, 'OPENID_USERINFO_PROVIDERS', None):
            settings.OPENID_USERINFO_PROVIDERS = \
                UserInfoProviderRegistry(getattr(settings, 'OPENID_CLAIM_PROVIDERS', []))

        if not hasattr(settings, 'OPENID_USER_CONSENT_VIEW'):
            settings.OPENID_USER_CONSENT_VIEW = 'openid_connect_op:consent'

        if not hasattr(settings, 'OPENID_DEFAULT_ACCESS_TOKEN_TTL'):
            settings.OPENID_DEFAULT_ACCESS_TOKEN_TTL = 3600

        if not hasattr(settings, 'OPENID_DEFAULT_REFRESH_TOKEN_TTL'):
            settings.OPENID_DEFAULT_REFRESH_TOKEN_TTL = 3600 * 24

        if not hasattr(settings, 'OPENID_CONNECT_OP_DB_ENCRYPT_KEY'):
            key = settings.SECRET_KEY
            while len(key)<16:
                key += key
            settings.OPENID_CONNECT_OP_DB_ENCRYPT_KEY = key.encode('utf-8')[:16]


import json
import re

from .crypto import CryptoTools


class ParameterType:
    def __init__(self, required=False, container_type=None, allowed_values=None):
        self.required = required
        self.container_type = container_type
        self.allowed_values = allowed_values

    def parse(self, parameter_name, parameter_value):
        if parameter_value is None:
            if self.required:
                raise AttributeError('Required parameter with name "%s" is not present' % parameter_name)
            else:
                parameter_value = ''
        if not self.container_type:
            return parameter_value

        if not isinstance(parameter_value, list):
            parameter_value = parameter_value.split()
        if self.allowed_values:
            for val in parameter_value:
                if val not in self.allowed_values:
                    raise AttributeError('Value "%s" is not allowed for parameter %s. Allowed values are %s'
                                         % (val, parameter_name,
                                            ', '.join(['"%s"' % x for x in sorted(self.allowed_values)])))
        if self.container_type is list:
            return parameter_value
        else:
            return self.container_type(parameter_value)

    def serialize(self, parameter_value):
        if self.container_type and parameter_value is not None:
            return list(parameter_value)
        else:
            return parameter_value

    def __eq__(self, other):
        if not isinstance(other, ParameterType):
            return False
        return self.required == other.required and \
               self.container_type == other.container_type and \
               self.allowed_values == other.allowed_values


class Parameters:
    """

    """

    """
    List of (param_name, ParameterType(...)) 
    """
    parameter_definitions = ()

    REQUIRED = ParameterType(required=True)
    OPTIONAL = ParameterType(required=False)
    ZLIB_DICT = b'openidhttp,,,,'

    def __init__(self, param_values):
        self.errors = []
        if param_values is not None:
            for parameter_name, parameter_definition in self.parameter_definitions:
                try:
                    parsed_value = parameter_definition.parse(parameter_name, param_values.get(parameter_name, None))
                    setattr(self, parameter_name, parsed_value)
                except AttributeError as e:
                    self.errors.append((parameter_name, e))

    def check_errors(self):
        if self.errors:
            raise self.errors[0][1]
        return self

    def to_dict(self):
        return {
            parameter_name: parameter_definition.serialize(getattr(self, parameter_name, None))
            for parameter_name, parameter_definition in self.parameter_definitions
        }

    def pack(self, encrypt=True, ttl=None, not_valid_before=None, key=None, prefix=b''):
        arr = []
        for d, dev in self.parameter_definitions:
            val = getattr(self, d)
            if isinstance(val, set) or isinstance(val, list) or isinstance(val, tuple):
                val = ' '.join(val)
            arr.append(str(val).replace('\\', '\\\\').replace(',', '\\,'))
        params = ','.join(arr)
        if not encrypt:
            return params
        return CryptoTools.encrypt(params.encode('utf-8'), ttl=ttl,
                                   not_valid_before=not_valid_before, key=key, prefix=prefix,
                                   zlib_dict=Parameters.ZLIB_DICT)

    @classmethod
    def unpack(cls, packed_data, decrypt=True, key=None, prefix=b''):
        if decrypt:
            params = CryptoTools.decrypt(packed_data, key=key, expected_prefix=prefix,
                                         zlib_dict=Parameters.ZLIB_DICT).decode('utf-8')
        else:
            params = packed_data
        params = re.split(r'(?<!\\),', params)
        param_dict = {}
        for d,p in zip(cls.parameter_definitions, params):
            val = p.replace('\\,', ',').replace('\\\\', '\\')
            param_dict[d[0]] = val

        return cls(param_dict)

    def __eq__(self, other):
        if not isinstance(other, Parameters):
            return False
        if self.parameter_definitions != other.parameter_definitions:
            return False
        for d, definition in self.parameter_definitions:
            if getattr(self, d) != getattr(other, d):
                return False
        return True

    def __str__(self):
        return json.dumps(self.to_dict(), sort_keys=True, default=lambda x: str(x))


from setuptools import setup, find_packages

README = """
# django-openid-op

This django application provides an implementation of OpenID Connect identity server
(OpenID provider). You can use it, for example, for building centralized authorization
server to which clients connect via the OpenID or OAuth2.0 protocol.

This library is compatible with python-social-auth package that can be used
as an OpenID client to access this server.

The following features of the OpenID Connect specification are implemented:

   * Basic profile from the OpenID Connect Core, including JWT signing
   * Subset of OpenID Connect Dynamic Registration
   * Subset of OpenID Content Discovery

For more details see https://github.com/mesemus/django-openid-op
"""

setup(
    name='django-openid-op',
    version='0.1.8',
    packages=find_packages(exclude='tests'),
    description='A django database based implementation of a subset of openid protocol, targeted at python3.6 and django 1.11+',
    long_description=README,
    author='Mirek Simek',
    author_email='miroslav.simek@gmail.com',
    url='https://github.com/mesemus/django-openid-op',
    license='MIT',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    install_requires=[
        'Django>=1.11',
        'pycryptodomex',
        'django-jsonfield',
        'django-ratelimit',
        'python-jwt',
        'django-modeltranslation',
        'django-bleach',
        'requests'
    ],
    tests_require=[
        'tox',
        'pytest',
        'pytest-django',
        'pytest_matrix',
        'pytest-runner',
        'pytest-env',
        'social-auth-app-django',
        'pyjwkest'
    ],
    extras_require={
        'dev': [
            'sphinx',
            'tox',
            'pytest',
            'pytest-django',
            'pytest_matrix',
            'pytest-runner',
            'pytest-env',
            'social-auth-app-django',
            'pyjwkest'
        ]
    }
)
